# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.azurestackhci import AzureStackHCIClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAzureStackHCIDeploymentSettingsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AzureStackHCIClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_list_by_clusters(self, resource_group):
        response = self.client.deployment_settings.list_by_clusters(
            resource_group_name=resource_group.name,
            cluster_name="str",
            api_version="2024-04-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_get(self, resource_group):
        response = self.client.deployment_settings.get(
            resource_group_name=resource_group.name,
            cluster_name="str",
            deployment_settings_name="default",
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_create_or_update(self, resource_group):
        response = self.client.deployment_settings.begin_create_or_update(
            resource_group_name=resource_group.name,
            cluster_name="str",
            resource={
                "arcNodeResourceIds": ["str"],
                "deploymentConfiguration": {
                    "scaleUnits": [
                        {
                            "deploymentData": {
                                "adouPath": "str",
                                "cluster": {
                                    "azureServiceEndpoint": "str",
                                    "cloudAccountName": "str",
                                    "name": "str",
                                    "witnessPath": "str",
                                    "witnessType": "str",
                                },
                                "domainFqdn": "str",
                                "hostNetwork": {
                                    "enableStorageAutoIp": False,
                                    "intents": [
                                        {
                                            "adapter": ["str"],
                                            "adapterPropertyOverrides": {
                                                "jumboPacket": "str",
                                                "networkDirect": "str",
                                                "networkDirectTechnology": "str",
                                            },
                                            "name": "str",
                                            "overrideAdapterProperty": False,
                                            "overrideQosPolicy": False,
                                            "overrideVirtualSwitchConfiguration": False,
                                            "qosPolicyOverrides": {
                                                "bandwidthPercentage_SMB": "str",
                                                "priorityValue8021Action_Cluster": "str",
                                                "priorityValue8021Action_SMB": "str",
                                            },
                                            "trafficType": ["str"],
                                            "virtualSwitchConfigurationOverrides": {
                                                "enableIov": "str",
                                                "loadBalancingAlgorithm": "str",
                                            },
                                        }
                                    ],
                                    "storageConnectivitySwitchless": False,
                                    "storageNetworks": [
                                        {
                                            "name": "str",
                                            "networkAdapterName": "str",
                                            "storageAdapterIPInfo": [
                                                {"ipv4Address": "str", "physicalNode": "str", "subnetMask": "str"}
                                            ],
                                            "vlanId": "str",
                                        }
                                    ],
                                },
                                "infrastructureNetwork": [
                                    {
                                        "dnsServers": ["str"],
                                        "gateway": "str",
                                        "ipPools": [{"endingAddress": "str", "startingAddress": "str"}],
                                        "subnetMask": "str",
                                        "useDhcp": bool,
                                    }
                                ],
                                "namingPrefix": "str",
                                "observability": {
                                    "episodicDataUpload": True,
                                    "euLocation": False,
                                    "streamingDataClient": True,
                                },
                                "optionalServices": {"customLocation": "str"},
                                "physicalNodes": [{"ipv4Address": "str", "name": "str"}],
                                "sdnIntegration": {
                                    "networkController": {
                                        "macAddressPoolStart": "str",
                                        "macAddressPoolStop": "str",
                                        "networkVirtualizationEnabled": bool,
                                    }
                                },
                                "secrets": [{"eceSecretName": "str", "secretLocation": "str", "secretName": "str"}],
                                "secretsLocation": "str",
                                "securitySettings": {
                                    "bitlockerBootVolume": True,
                                    "bitlockerDataVolumes": True,
                                    "credentialGuardEnforced": False,
                                    "driftControlEnforced": True,
                                    "drtmProtection": True,
                                    "hvciProtection": True,
                                    "sideChannelMitigationEnforced": True,
                                    "smbClusterEncryption": False,
                                    "smbSigningEnforced": True,
                                    "wdacEnforced": True,
                                },
                                "storage": {"configurationMode": "Express"},
                            },
                            "sbePartnerInfo": {
                                "credentialList": [
                                    {"eceSecretName": "str", "secretLocation": "str", "secretName": "str"}
                                ],
                                "partnerProperties": [{"name": "str", "value": "str"}],
                                "sbeDeploymentInfo": {
                                    "family": "str",
                                    "publisher": "str",
                                    "sbeManifestCreationDate": "2020-02-20 00:00:00",
                                    "sbeManifestSource": "str",
                                    "version": "str",
                                },
                            },
                        }
                    ],
                    "version": "str",
                },
                "deploymentMode": "Deploy",
                "id": "str",
                "name": "str",
                "operationType": "ClusterProvisioning",
                "provisioningState": "str",
                "reportedProperties": {
                    "deploymentStatus": {
                        "status": "str",
                        "steps": [
                            {
                                "description": "str",
                                "endTimeUtc": "str",
                                "exception": ["str"],
                                "fullStepIndex": "str",
                                "name": "str",
                                "startTimeUtc": "str",
                                "status": "str",
                                "steps": [...],
                            }
                        ],
                    },
                    "validationStatus": {
                        "status": "str",
                        "steps": [
                            {
                                "description": "str",
                                "endTimeUtc": "str",
                                "exception": ["str"],
                                "fullStepIndex": "str",
                                "name": "str",
                                "startTimeUtc": "str",
                                "status": "str",
                                "steps": [...],
                            }
                        ],
                    },
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
            deployment_settings_name="default",
            api_version="2024-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_begin_delete(self, resource_group):
        response = self.client.deployment_settings.begin_delete(
            resource_group_name=resource_group.name,
            cluster_name="str",
            deployment_settings_name="default",
            api_version="2024-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from time import time
from wsgiref.handlers import format_date_time
from urllib.parse import quote

from corehttp.rest import HttpRequest
from corehttp.exceptions import (
    HttpResponseError,
    map_error,
)
from ._test_base import _TableTest


class QueryEntitiesJSONTest(_TableTest):
    def __init__(self, arguments):
        super().__init__(arguments)
        # query params
        self.select = quote("Property2")
        self.filter = quote("Property1 eq 'a'")
        self.url = f"{self.account_endpoint}{self.table_name}"

    async def global_setup(self):
        await super().global_setup()
        # create entity to be queried
        entity = self.get_entity()
        await self.async_table_client.create_entity(entity)

    def run_sync(self):
        current_time = format_date_time(time())
        request = HttpRequest(
            method="GET",
            url=self.url,
            params={"$select": self.select, "$filter": self.filter},
            headers={
                "x-ms-version": self.api_version,
                "DataServiceVersion": self.data_service_version,
                "Content-Type": "application/json",
                "Accept": "application/json",
                "x-ms-date": current_time,
            },
        )
        # Many policies in Azure SDKs use the backcompatible attribute `query` on HttpRequest. This is not present in corehttp.HttpRequest, so we add it manually to make
        # Azure SDK policies work with corehttp.
        request.query = {"$select": self.select, "$filter": self.filter}
        response = self.pipeline_client.pipeline.run(
            request,
        ).http_response
        response.json()
        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=self.error_map)
            raise HttpResponseError(response=response)

    async def run_async(self):
        current_time = format_date_time(time())
        request = HttpRequest(
            method="GET",
            url=self.url,
            params={"$select": self.select, "$filter": self.filter},
            headers={
                "x-ms-version": self.api_version,
                "DataServiceVersion": self.data_service_version,
                "Content-Type": "application/json",
                "Accept": "application/json",
                "x-ms-date": current_time,
            },
        )
        # Many policies in Azure SDKs use the backcompatible attribute `query` on HttpRequest. This is not present in corehttp.HttpRequest, so we add it manually to make
        # Azure SDK policies work with corehttp.
        request.query = {"$select": self.select, "$filter": self.filter}
        response = (
            await self.async_pipeline_client.pipeline.run(
                request,
            )
        ).http_response
        response.json()
        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=self.error_map)
            raise HttpResponseError(response=response)

    async def close(self):
        await self.async_table_client.close()
        await super().close()



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    Given an AIProjectClient, this sample demonstrates how to use the synchronous
    `.evaluations` methods to submit evaluation for an agent run.

USAGE:
    python sample_agent_evaluations.py

    Before running the sample:

    pip install azure-ai-projects azure-identity

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - Required. The Azure AI Project endpoint, as found in the overview page of your
       Azure AI Foundry project.
"""

import os, time

from azure.identity import DefaultAzureCredential
from azure.ai.projects import AIProjectClient
from azure.ai.projects.models import (
    AgentEvaluationRequest,
    InputDataset,
    EvaluatorIds,
    EvaluatorConfiguration,
    AgentEvaluationSamplingConfiguration,
    AgentEvaluationRedactionConfiguration,
)
from dotenv import load_dotenv

load_dotenv()

endpoint = os.environ["PROJECT_ENDPOINT"]
model_deployment_name = os.environ["MODEL_DEPLOYMENT_NAME"]


with DefaultAzureCredential(exclude_interactive_browser_credential=False) as credential:

    # TODO : Remove api_version when the service is stable
    with AIProjectClient(endpoint=endpoint, credential=credential) as project_client:

        # [START evaluations_agent_sample]
        agent = project_client.agents.create_agent(
            model=model_deployment_name,
            name="my-agent",
            instructions="You are helpful agent",
        )
        print(f"Created agent, agent ID: {agent.id}")

        thread = project_client.agents.threads.create()
        print(f"Created thread, thread ID: {thread.id}")

        message = project_client.agents.messages.create(
            thread_id=thread.id, role="user", content="Hello, tell me a joke"
        )
        print(f"Created message, message ID: {message.id}")

        run = project_client.agents.runs.create(thread_id=thread.id, agent_id=agent.id)

        # Poll the run as long as run status is queued or in progress
        while run.status in ["queued", "in_progress", "requires_action"]:
            # Wait for a second
            time.sleep(1)
            run = project_client.agents.runs.get(thread_id=thread.id, run_id=run.id)
            print(f"Run status: {run.status}")

        agent_evaluation_request = AgentEvaluationRequest(
            run_id=run.id,
            thread_id=thread.id,
            evaluators={
                "violence": EvaluatorConfiguration(
                    id=EvaluatorIds.VIOLENCE,
                )
            },
            sampling_configuration=AgentEvaluationSamplingConfiguration(
                name="test",
                sampling_percent=100,
                max_request_rate=100,
            ),
            redaction_configuration=AgentEvaluationRedactionConfiguration(
                redact_score_properties=False,
            ),
            app_insights_connection_string=project_client.telemetry.get_connection_string(),
        )

        agent_evaluation_response = project_client.evaluations.create_agent_evaluation(
            evaluation=agent_evaluation_request
        )

        print(agent_evaluation_response)

        # [END evaluations_agent_sample]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# This script is used to execute ruff within a tox environment.

import subprocess
import argparse
import os
import sys
from ci_tools.parsing import ParsedSetup


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Run ruff against target folder."
    )

    parser.add_argument(
        "-t",
        "--target",
        dest="target_package",
        help="The target package directory on disk. The target module passed to ruff will be <target_package>/azure.",
        required=True,
    )

    args = parser.parse_args()

    pkg_dir = os.path.abspath(args.target_package)
    pkg_details = ParsedSetup.from_path(pkg_dir)
    top_level_module = pkg_details.namespace.split('.')[0]

    subprocess.check_call(
        [
            sys.executable,
            "-m",
            "ruff",
            "check",
            os.path.join(args.target_package, top_level_module),
        ]
    )



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._bots_operations import BotsOperations
from ._channels_operations import ChannelsOperations
from ._direct_line_operations import DirectLineOperations
from ._email_operations import EmailOperations
from ._operations import Operations
from ._bot_connection_operations import BotConnectionOperations
from ._qn_amaker_endpoint_keys_operations import QnAMakerEndpointKeysOperations
from ._host_settings_operations import HostSettingsOperations
from ._operation_results_operations import OperationResultsOperations
from ._private_endpoint_connections_operations import PrivateEndpointConnectionsOperations
from ._private_link_resources_operations import PrivateLinkResourcesOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "BotsOperations",
    "ChannelsOperations",
    "DirectLineOperations",
    "EmailOperations",
    "Operations",
    "BotConnectionOperations",
    "QnAMakerEndpointKeysOperations",
    "HostSettingsOperations",
    "OperationResultsOperations",
    "PrivateEndpointConnectionsOperations",
    "PrivateLinkResourcesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# This script is intended to be a place holder for common tasks that are requried by scripts running on tox

import shutil
import sys
import logging
import os
import glob
import zipfile
import tarfile
import subprocess
import re

logging.getLogger().setLevel(logging.INFO)

def unzip_sdist_to_directory(containing_folder: str) -> str:
    zips = glob.glob(os.path.join(containing_folder, "*.zip"))

    if zips:
        return unzip_file_to_directory(zips[0], containing_folder)
    else:
        tars = glob.glob(os.path.join(containing_folder, "*.tar.gz"))
        return unzip_file_to_directory(tars[0], containing_folder)


def unzip_file_to_directory(path_to_zip_file: str, extract_location: str) -> str:
    if path_to_zip_file.endswith(".zip"):
        with zipfile.ZipFile(path_to_zip_file, "r") as zip_ref:
            zip_ref.extractall(extract_location)
            extracted_dir = os.path.basename(os.path.splitext(path_to_zip_file)[0])
            return os.path.join(extract_location, extracted_dir)
    else:
        with tarfile.open(path_to_zip_file) as tar_ref:
            tar_ref.extractall(extract_location)
            extracted_dir = os.path.basename(path_to_zip_file).replace(".tar.gz", "")
            return os.path.join(extract_location, extracted_dir)


def move_and_rename(source_location):
    new_location = os.path.join(os.path.dirname(source_location), "unzipped")

    if os.path.exists(new_location):
        shutil.rmtree(new_location)

    os.rename(source_location, new_location)
    return new_location



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from enum import Enum
from typing import Any, Dict
import jsondiff
from breaking_changes_tracker import BreakingChangesTracker

class ChangeType(str, Enum):
    ADDED_CLIENT = "AddedClient"
    ADDED_CLIENT_METHOD = "AddedClientMethod"
    ADDED_CLASS = "AddedClass"
    ADDED_CLASS_METHOD = "AddedClassMethod"
    ADDED_CLASS_METHOD_PARAMETER = "AddedClassMethodParameter"
    ADDED_CLASS_PROPERTY = "AddedClassProperty"
    ADDED_ENUM = "AddedEnum"
    ADDED_ENUM_MEMBER = "AddedEnumMember"
    ADDED_FUNCTION_PARAMETER = "AddedFunctionParameter"
    ADDED_OPERATION_GROUP = "AddedOperationGroup"

class ChangelogTracker(BreakingChangesTracker):
    ADDED_CLIENT_MSG = \
        "Added client `{}`"
    ADDED_CLIENT_METHOD_MSG = \
        "Client `{}` added method `{}`"
    ADDED_CLASS_MSG = \
        "Added model `{}`"
    ADDED_CLASS_METHOD_MSG = \
        "Model `{}` added method `{}`"
    ADDED_CLASS_METHOD_PARAMETER_MSG = \
        "Model `{}` added parameter `{}` in method `{}`"
    ADDED_FUNCTION_PARAMETER_MSG = \
        "Function `{}` added parameter `{}`"
    ADDED_CLASS_PROPERTY_MSG = \
        "Model `{}` added property `{}`"
    ADDED_ENUM_MSG = \
        "Added enum `{}`"
    ADDED_ENUM_MEMBER_MSG = \
        "Enum `{}` added member `{}`"
    ADDED_OPERATION_GROUP_MSG = \
        "Client `{}` added operation group `{}`"


    def __init__(self, stable: Dict, current: Dict, package_name: str, **kwargs: Any) -> None:
        super().__init__(stable, current, package_name, **kwargs)
        self.features_added = []

    def run_checks(self) -> None:
        self.run_non_breaking_change_diff_checks()
        super().run_checks()
        self.run_async_cleanup(self.features_added)

    def run_non_breaking_change_diff_checks(self) -> None:
        for module_name, module in self.diff.items():
            self.module_name = module_name
            if self.module_name not in self.stable and not isinstance(self.module_name, jsondiff.Symbol):
                continue  # TODO add this to reported changes

            self.run_non_breaking_class_level_diff_checks(module)

    def run_non_breaking_class_level_diff_checks(self, module: Dict) -> None:
        for class_name, class_components in module.get("class_nodes", {}).items():
            self.class_name = class_name
            stable_class_nodes = self.stable[self.module_name]["class_nodes"]
            if not isinstance(class_name, jsondiff.Symbol):
                if self.class_name not in stable_class_nodes:
                    if self.class_name.endswith("Client"):
                        # This is a new client
                        fa = (
                            self.ADDED_CLIENT_MSG,
                            ChangeType.ADDED_CLIENT,
                            self.module_name, class_name
                        )
                        self.features_added.append(fa)
                    elif class_components.get("type", None) == "Enum":
                        # This is a new enum
                        fa = (
                            self.ADDED_ENUM_MSG,
                            ChangeType.ADDED_ENUM,
                            self.module_name, class_name
                        )
                        self.features_added.append(fa)
                    else:
                        # This is a new class
                        fa = (
                            self.ADDED_CLASS_MSG,
                            ChangeType.ADDED_CLASS,
                            self.module_name, class_name
                        )
                        self.features_added.append(fa)
                else:
                    # Check existing class for new methods
                    stable_methods_node = stable_class_nodes[self.class_name]["methods"]
                    for method_name, method_components in class_components.get("methods", {}).items():
                        self.function_name = method_name
                        if not isinstance(self.function_name, jsondiff.Symbol):
                            if self.function_name not in stable_methods_node:
                                if self.class_name.endswith("Client"):
                                    # This is a new client method
                                    fa = (
                                        self.ADDED_CLIENT_METHOD_MSG,
                                        ChangeType.ADDED_CLIENT_METHOD,
                                        self.module_name, self.class_name, method_name
                                    )
                                    self.features_added.append(fa)
                                else:
                                    # This is a new class method
                                    fa = (
                                        self.ADDED_CLASS_METHOD_MSG,
                                        ChangeType.ADDED_CLASS_METHOD,
                                        self.module_name, class_name, method_name
                                    )
                                    self.features_added.append(fa)
                            else:
                                # Check existing methods for new parameters
                                stable_parameters_node = stable_methods_node[self.function_name]["parameters"]
                                current_parameters_node = self.current[self.module_name]["class_nodes"][self.class_name]["methods"][self.function_name]["parameters"]
                                for param_name, param_components in method_components.get("parameters", {}).items():
                                    self.parameter_name = param_name
                                    if self.parameter_name not in stable_parameters_node and \
                                            not isinstance(self.parameter_name, jsondiff.Symbol):
                                        if self.function_name == "__init__":
                                            # If this is a new class property skip reporting it here and let the class properties check handle it.
                                            # This is because we'll get multiple reports for the same new property if it's a parameter in __init__
                                            # and a class level attribute.
                                            if self.parameter_name in class_components.get("properties", {}).keys():
                                                continue
                                        self.check_non_positional_parameter_added(
                                            current_parameters_node[param_name]
                                        )
                    stable_property_node = stable_class_nodes[self.class_name]["properties"]
                    for property_name, property_components in class_components.get("properties", {}).items():
                        if property_name not in stable_property_node and \
                                not isinstance(property_name, jsondiff.Symbol):
                            fa = (
                                    self.ADDED_CLASS_PROPERTY_MSG,
                                    ChangeType.ADDED_CLASS_PROPERTY,
                                    self.module_name, class_name, property_name
                                )
                            if self.class_name.endswith("Client"):
                                if property_components["attr_type"] is not None and property_components["attr_type"].lower().endswith("operations"):
                                    fa = (
                                        self.ADDED_OPERATION_GROUP_MSG,
                                        ChangeType.ADDED_OPERATION_GROUP,
                                        self.module_name, self.class_name, property_name
                                    )
                            if stable_class_nodes[self.class_name]["type"] == "Enum":
                                fa = (
                                    self.ADDED_ENUM_MEMBER_MSG,
                                    ChangeType.ADDED_ENUM_MEMBER,
                                    self.module_name, class_name, property_name
                                )
                            self.features_added.append(fa)


    def check_non_positional_parameter_added(self, current_parameters_node: Dict) -> None:
        if current_parameters_node["param_type"] != "positional_or_keyword":
            if self.class_name:
                self.features_added.append(
                    (
                        self.ADDED_CLASS_METHOD_PARAMETER_MSG, ChangeType.ADDED_CLASS_METHOD_PARAMETER,
                        self.module_name, self.class_name, self.parameter_name, self.function_name
                    )
                )
            else:
                self.features_added.append(
                    (
                        self.ADDED_FUNCTION_PARAMETER_MSG, ChangeType.ADDED_FUNCTION_PARAMETER,
                        self.module_name, self.function_name, self.parameter_name
                    )
                )


    def report_changes(self) -> None:
        ignore_changes = self.ignore if self.ignore else {}
        self.get_reportable_changes(ignore_changes, self.breaking_changes)
        self.get_reportable_changes(ignore_changes, self.features_added)
        # Code borrowed and modified from the previous change log tool
        def _build_md(content: list, title: str, buffer: list):
            buffer.append(title)
            buffer.append("")
            for _, bc in enumerate(content):
                # Extract the message, skip the change type and the module name
                msg, _, _,*args = bc
                buffer.append("  - " + msg.format(*args))
            buffer.append("")
            return buffer

        buffer = []
        if self.features_added:
            _build_md(self.features_added, "### Features Added", buffer)
        if self.breaking_changes:
            _build_md(self.breaking_changes, "### Breaking Changes", buffer)
        content =  "\n".join(buffer).strip()

        content = "===== changelog start =====\n" + content + "\n===== changelog end =====\n"
        return content



import logging
import os
import sys
import time
import unittest
import uuid
from typing import Callable

import pytest
from azure.core.pipeline.transport._requests_basic import RequestsTransport
from azure.core.rest import HttpRequest

import test_config
from azure.cosmos import PartitionKey
from azure.cosmos import CosmosClient
from azure.cosmos.container import ContainerProxy
from azure.cosmos.database import DatabaseProxy
from azure.cosmos.exceptions import CosmosHttpResponseError
from _fault_injection_transport import FaultInjectionTransport
from azure.core.exceptions import ServiceRequestError

logger = logging.getLogger('azure.cosmos')
logger.setLevel(logging.DEBUG)
logger.addHandler(logging.StreamHandler(sys.stdout))

host = test_config.TestConfig.host
master_key = test_config.TestConfig.masterKey
TEST_DATABASE_ID = test_config.TestConfig.TEST_DATABASE_ID
SINGLE_PARTITION_CONTAINER_NAME = os.path.basename(__file__) + str(uuid.uuid4())

@pytest.mark.unittest
@pytest.mark.cosmosEmulator
class TestFaultInjectionTransport:

    @classmethod
    def setup_class(cls):
        logger.info("starting class: {} execution".format(cls.__name__))
        cls.host = host
        cls.master_key = master_key

        if (cls.master_key == '[YOUR_KEY_HERE]' or
                cls.host == '[YOUR_ENDPOINT_HERE]'):
            raise Exception(
                "You must specify your Azure Cosmos account values for "
                "'masterKey' and 'host' at the top of this class to run the "
                "tests.")
        cls.database_id = TEST_DATABASE_ID
        cls.single_partition_container_name = SINGLE_PARTITION_CONTAINER_NAME

        cls.mgmt_client = CosmosClient(cls.host, cls.master_key, consistency_level="Session", logger=logger)
        created_database = cls.mgmt_client.get_database_client(cls.database_id)
        created_database.create_container(cls.single_partition_container_name, partition_key=PartitionKey("/pk"))


    @classmethod
    def teardown_class(cls):
        logger.info("tearing down class: {}".format(cls.__name__))
        created_database = cls.mgmt_client.get_database_client(cls.database_id)
        try:
            created_database.delete_container(cls.single_partition_container_name),
        except Exception as containerDeleteError:
            logger.warning("Exception trying to delete database {}. {}".format(created_database.id, containerDeleteError))

    @staticmethod
    def setup_method_with_custom_transport(
            custom_transport: RequestsTransport,
            default_endpoint: str = host,
            key: str = master_key,
            database_id: str = TEST_DATABASE_ID,
            container_id: str = SINGLE_PARTITION_CONTAINER_NAME,
            custom_logger = logger,
            **kwargs):
        client = CosmosClient(default_endpoint, key, consistency_level="Session",
                              transport=custom_transport, logger=custom_logger, enable_diagnostics_logging=True, **kwargs)
        db: DatabaseProxy = client.get_database_client(database_id)
        container: ContainerProxy = db.get_container_client(container_id)
        return {"client": client, "db": db, "col": container}


    def test_throws_injected_error(self: "TestFaultInjectionTransport"):
        id_value: str = str(uuid.uuid4())
        document_definition = {'id': id_value,
                               'pk': id_value,
                               'name': 'sample document',
                               'key': 'value'}

        custom_transport =  FaultInjectionTransport()
        predicate : Callable[[HttpRequest], bool] = lambda r: FaultInjectionTransport.predicate_req_for_document_with_id(r, id_value)
        custom_transport.add_fault(predicate, lambda r: FaultInjectionTransport.error_after_delay(
            10000,
            CosmosHttpResponseError(
                status_code=502,
                message="Some random reverse proxy error.")))

        initialized_objects = TestFaultInjectionTransport.setup_method_with_custom_transport(custom_transport)
        start: float = time.perf_counter()
        try:
            container: ContainerProxy = initialized_objects["col"]
            container.create_item(body=document_definition)
            pytest.fail("Expected exception not thrown")
        except CosmosHttpResponseError as cosmosError:
            end = time.perf_counter() - start
            # validate response took more than 10 seconds
            assert end > 10
            if cosmosError.status_code != 502:
                raise cosmosError

    def test_swr_mrr_succeeds(self: "TestFaultInjectionTransport"):
        expected_read_region_uri: str = test_config.TestConfig.local_host
        expected_write_region_uri: str = expected_read_region_uri.replace("localhost", "127.0.0.1")
        custom_transport = FaultInjectionTransport()
        # Inject rule to disallow writes in the read-only region
        is_write_operation_in_read_region_predicate: Callable[[HttpRequest], bool] = lambda \
            r: FaultInjectionTransport.predicate_is_write_operation(r, expected_read_region_uri)

        custom_transport.add_fault(
            is_write_operation_in_read_region_predicate,
            lambda r: FaultInjectionTransport.error_write_forbidden())

        # Inject topology transformation that would make Emulator look like a single write region
        # account with two read regions
        is_get_account_predicate: Callable[[HttpRequest], bool] = lambda r: FaultInjectionTransport.predicate_is_database_account_call(r)
        emulator_as_multi_region_sm_account_transformation = \
            lambda r, inner: FaultInjectionTransport.transform_topology_swr_mrr(
                write_region_name="Write Region",
                read_region_name="Read Region",
                inner=inner)
        custom_transport.add_response_transformation(
            is_get_account_predicate,
            emulator_as_multi_region_sm_account_transformation)

        id_value: str = str(uuid.uuid4())
        document_definition = {'id': id_value,
                               'pk': id_value,
                               'name': 'sample document',
                               'key': 'value'}

        initialized_objects = self.setup_method_with_custom_transport(
            custom_transport,
            preferred_locations=["Read Region", "Write Region"])
        container: ContainerProxy = initialized_objects["col"]

        created_document = container.create_item(body=document_definition)
        request: HttpRequest = created_document.get_response_headers()["_request"]
        # Validate the response comes from "Write Region" (the write region)
        assert request.url.startswith(expected_write_region_uri)
        start: float = time.perf_counter()

        while (time.perf_counter() - start) < 2:
            read_document = container.read_item(id_value, partition_key=id_value)
            request = read_document.get_response_headers()["_request"]
            # Validate the response comes from "Read Region" (the most preferred read-only region)
            assert request.url.startswith(expected_read_region_uri)


    def test_swr_mrr_region_down_read_succeeds(self: "TestFaultInjectionTransport"):
        expected_read_region_uri: str = test_config.TestConfig.local_host
        expected_write_region_uri: str = expected_read_region_uri.replace("localhost", "127.0.0.1")
        custom_transport = FaultInjectionTransport()
        # Inject rule to disallow writes in the read-only region
        is_write_operation_in_read_region_predicate: Callable[[HttpRequest], bool] = lambda \
            r: FaultInjectionTransport.predicate_is_write_operation(r, expected_read_region_uri)

        custom_transport.add_fault(
            is_write_operation_in_read_region_predicate,
            lambda r: FaultInjectionTransport.error_write_forbidden())

        # Inject rule to simulate regional outage in "Read Region"
        is_request_to_read_region: Callable[[HttpRequest], bool] = lambda \
                r: FaultInjectionTransport.predicate_targets_region(r, expected_read_region_uri)

        custom_transport.add_fault(
            is_request_to_read_region,
            lambda r: FaultInjectionTransport.error_region_down())

        # Inject topology transformation that would make Emulator look like a single write region
        # account with two read regions
        is_get_account_predicate: Callable[[HttpRequest], bool] = lambda r: FaultInjectionTransport.predicate_is_database_account_call(r)
        emulator_as_multi_region_sm_account_transformation = \
            lambda r, inner: FaultInjectionTransport.transform_topology_swr_mrr(
                write_region_name="Write Region",
                read_region_name="Read Region",
                inner=inner)
        custom_transport.add_response_transformation(
            is_get_account_predicate,
            emulator_as_multi_region_sm_account_transformation)

        id_value: str = str(uuid.uuid4())
        document_definition = {'id': id_value,
                               'pk': id_value,
                               'name': 'sample document',
                               'key': 'value'}

        initialized_objects = self.setup_method_with_custom_transport(
            custom_transport,
            default_endpoint=expected_write_region_uri,
            preferred_locations=["Read Region", "Write Region"])
        container: ContainerProxy = initialized_objects["col"]

        created_document = container.create_item(body=document_definition)
        request: HttpRequest = created_document.get_response_headers()["_request"]
        # Validate the response comes from "South Central US" (the write region)
        assert request.url.startswith(expected_write_region_uri)
        start:float = time.perf_counter()

        while (time.perf_counter() - start) < 2:
            read_document = container.read_item(id_value, partition_key=id_value)
            request = read_document.get_response_headers()["_request"]
            # Validate the response comes from "Write Region" ("Read Region" the most preferred read-only region is down)
            assert request.url.startswith(expected_write_region_uri)


    def test_swr_mrr_region_down_envoy_read_succeeds(self: "TestFaultInjectionTransport"):
        expected_read_region_uri: str = test_config.TestConfig.local_host
        expected_write_region_uri: str = expected_read_region_uri.replace("localhost", "127.0.0.1")
        custom_transport = FaultInjectionTransport()
        # Inject rule to disallow writes in the read-only region
        is_write_operation_in_read_region_predicate: Callable[[HttpRequest], bool] = lambda \
            r: FaultInjectionTransport.predicate_is_write_operation(r, expected_read_region_uri)

        custom_transport.add_fault(
            is_write_operation_in_read_region_predicate,
            lambda r: FaultInjectionTransport.error_write_forbidden())

        # Inject rule to simulate regional outage in "Read Region"
        is_request_to_read_region: Callable[[HttpRequest], bool] = lambda \
                r: FaultInjectionTransport.predicate_targets_region(r, expected_read_region_uri) and \
                    FaultInjectionTransport.predicate_is_document_operation(r)

        custom_transport.add_fault(
            is_request_to_read_region,
            lambda r: FaultInjectionTransport.error_after_delay(
                500,
                CosmosHttpResponseError(
                    status_code=502,
                    message="Some random reverse proxy error.")))

        # Inject topology transformation that would make Emulator look like a single write region
        # account with two read regions
        is_get_account_predicate: Callable[[HttpRequest], bool] = lambda r: FaultInjectionTransport.predicate_is_database_account_call(r)
        emulator_as_multi_region_sm_account_transformation = \
            lambda r, inner: FaultInjectionTransport.transform_topology_swr_mrr(
                write_region_name="Write Region",
                read_region_name="Read Region",
                inner=inner)
        custom_transport.add_response_transformation(
            is_get_account_predicate,
            emulator_as_multi_region_sm_account_transformation)

        id_value: str = str(uuid.uuid4())
        document_definition = {'id': id_value,
                               'pk': id_value,
                               'name': 'sample document',
                               'key': 'value'}

        initialized_objects = self.setup_method_with_custom_transport(
        custom_transport,
        default_endpoint=expected_write_region_uri,
        preferred_locations=["Read Region", "Write Region"])
        container: ContainerProxy = initialized_objects["col"]

        created_document = container.create_item(body=document_definition)
        request: HttpRequest = created_document.get_response_headers()["_request"]
        # Validate the response comes from "South Central US" (the write region)
        assert request.url.startswith(expected_write_region_uri)
        start:float = time.perf_counter()

        while (time.perf_counter() - start) < 2:
            read_document = container.read_item(id_value, partition_key=id_value)
            request = read_document.get_response_headers()["_request"]
            # Validate the response comes from "Write Region" ("Read Region" the most preferred read-only region is down)
            assert request.url.startswith(expected_write_region_uri)



    def test_mwr_succeeds(self: "TestFaultInjectionTransport"):
        first_region_uri: str = test_config.TestConfig.local_host.replace("localhost", "127.0.0.1")
        custom_transport = FaultInjectionTransport()

        # Inject topology transformation that would make Emulator look like a multiple write region account
        # account with two read regions
        is_get_account_predicate: Callable[[HttpRequest], bool] = lambda r: FaultInjectionTransport.predicate_is_database_account_call(r)
        emulator_as_multi_write_region_account_transformation = \
            lambda r, inner: FaultInjectionTransport.transform_topology_mwr(
                first_region_name="First Region",
                second_region_name="Second Region",
                inner=inner)
        custom_transport.add_response_transformation(
            is_get_account_predicate,
            emulator_as_multi_write_region_account_transformation)

        id_value: str = str(uuid.uuid4())
        document_definition = {'id': id_value,
                               'pk': id_value,
                               'name': 'sample document',
                               'key': 'value'}

        initialized_objects = self.setup_method_with_custom_transport(
            custom_transport,
            preferred_locations=["First Region", "Second Region"],
            multiple_write_locations=True
        )
        container: ContainerProxy = initialized_objects["col"]

        created_document = container.create_item(body=document_definition)
        request: HttpRequest = created_document.get_response_headers()["_request"]
        # Validate the response comes from "South Central US" (the write region)
        assert request.url.startswith(first_region_uri)
        start:float = time.perf_counter()

        while (time.perf_counter() - start) < 2:
            read_document = container.read_item(id_value, partition_key=id_value)
            request = read_document.get_response_headers()["_request"]
            # Validate the response comes from "East US" (the most preferred read-only region)
            assert request.url.startswith(first_region_uri)


    def test_mwr_region_down_succeeds(self: "TestFaultInjectionTransport"):
        first_region_uri: str = test_config.TestConfig.local_host.replace("localhost", "127.0.0.1")
        second_region_uri: str = test_config.TestConfig.local_host
        custom_transport = FaultInjectionTransport()

        # Inject topology transformation that would make Emulator look like a multiple write region account
        # account with two read regions
        is_get_account_predicate: Callable[[HttpRequest], bool] = lambda r: FaultInjectionTransport.predicate_is_database_account_call(r)
        emulator_as_multi_write_region_account_transformation = \
            lambda r, inner: FaultInjectionTransport.transform_topology_mwr(
                first_region_name="First Region",
                second_region_name="Second Region",
                inner=inner)
        custom_transport.add_response_transformation(
            is_get_account_predicate,
            emulator_as_multi_write_region_account_transformation)

        # Inject rule to simulate regional outage in "First Region"
        is_request_to_first_region: Callable[[HttpRequest], bool] = lambda \
                r: FaultInjectionTransport.predicate_targets_region(r, first_region_uri) and \
                   FaultInjectionTransport.predicate_is_document_operation(r)

        custom_transport.add_fault(
            is_request_to_first_region,
            lambda r: FaultInjectionTransport.error_region_down())

        id_value: str = str(uuid.uuid4())
        document_definition = {'id': id_value,
                               'pk': id_value,
                               'name': 'sample document',
                               'key': 'value'}

        initialized_objects = self.setup_method_with_custom_transport(
            custom_transport,
            preferred_locations=["First Region", "Second Region"],
            multiple_write_locations=True
        )
        container: ContainerProxy = initialized_objects["col"]

        start:float = time.perf_counter()
        while (time.perf_counter() - start) < 2:
            # reads and writes should failover to second region
            upsert_document = container.upsert_item(body=document_definition)
            request = upsert_document.get_response_headers()["_request"]
            assert request.url.startswith(second_region_uri)
            read_document = container.read_item(id_value, partition_key=id_value)
            request = read_document.get_response_headers()["_request"]
            # Validate the response comes from "East US" (the most preferred read-only region)
            assert request.url.startswith(second_region_uri)


    def test_swr_mrr_all_regions_down_for_read(self: "TestFaultInjectionTransport"):
        expected_read_region_uri: str = test_config.TestConfig.local_host
        expected_write_region_uri: str = expected_read_region_uri.replace("localhost", "127.0.0.1")
        custom_transport = FaultInjectionTransport()

        # Inject rule to disallow writes in the read-only region
        is_write_operation_in_read_region_predicate: Callable[[HttpRequest], bool] = lambda \
                r: FaultInjectionTransport.predicate_is_write_operation(r, expected_read_region_uri)

        custom_transport.add_fault(
            is_write_operation_in_read_region_predicate,
            lambda r: FaultInjectionTransport.error_write_forbidden())

        # Inject topology transformation that would make Emulator look like a multiple write region account
        # account with two read regions
        is_get_account_predicate: Callable[[HttpRequest], bool] = lambda \
            r: FaultInjectionTransport.predicate_is_database_account_call(r)
        emulator_as_multi_write_region_account_transformation = \
            lambda r, inner: FaultInjectionTransport.transform_topology_swr_mrr(
                write_region_name="Write Region",
                read_region_name="Read Region",
                inner=inner)
        custom_transport.add_response_transformation(
            is_get_account_predicate,
            emulator_as_multi_write_region_account_transformation)

        # Inject rule to simulate regional outage in "First Region"
        is_request_to_first_region: Callable[[HttpRequest], bool] = lambda \
                r: (FaultInjectionTransport.predicate_targets_region(r, expected_write_region_uri) and
                   FaultInjectionTransport.predicate_is_document_operation(r) and
                    not FaultInjectionTransport.predicate_is_write_operation(r, expected_write_region_uri))


        # Inject rule to simulate regional outage in "Second Region"
        is_request_to_second_region: Callable[[HttpRequest], bool] = lambda \
                r: (FaultInjectionTransport.predicate_targets_region(r, expected_read_region_uri) and
                   FaultInjectionTransport.predicate_is_document_operation(r) and
                    not FaultInjectionTransport.predicate_is_write_operation(r, expected_write_region_uri))

        custom_transport.add_fault(
            is_request_to_first_region,
            lambda r: FaultInjectionTransport.error_region_down())
        custom_transport.add_fault(
            is_request_to_second_region,
            lambda r: FaultInjectionTransport.error_region_down())

        id_value: str = str(uuid.uuid4())
        document_definition = {'id': id_value,
                               'pk': id_value,
                               'name': 'sample document',
                               'key': 'value'}

        initialized_objects = self.setup_method_with_custom_transport(
            custom_transport,
            preferred_locations=["First Region", "Second Region"])
        container: ContainerProxy = initialized_objects["col"]
        container.upsert_item(body=document_definition)
        with pytest.raises(ServiceRequestError):
            container.read_item(id_value, partition_key=id_value)

    def test_mwr_all_regions_down(self: "TestFaultInjectionTransport"):

        first_region_uri: str = test_config.TestConfig.local_host.replace("localhost", "127.0.0.1")
        second_region_uri: str = test_config.TestConfig.local_host
        custom_transport = FaultInjectionTransport()

        # Inject topology transformation that would make Emulator look like a multiple write region account
        # account with two read regions
        is_get_account_predicate: Callable[[HttpRequest], bool] = lambda \
                r: FaultInjectionTransport.predicate_is_database_account_call(r)
        emulator_as_multi_write_region_account_transformation = \
            lambda r, inner: FaultInjectionTransport.transform_topology_mwr(
                first_region_name="First Region",
                second_region_name="Second Region",
                inner=inner)
        custom_transport.add_response_transformation(
            is_get_account_predicate,
            emulator_as_multi_write_region_account_transformation)

        # Inject rule to simulate regional outage in "First Region"
        is_request_to_first_region: Callable[[HttpRequest], bool] = lambda \
                r: FaultInjectionTransport.predicate_targets_region(r, first_region_uri) and \
                   FaultInjectionTransport.predicate_is_document_operation(r)

        # Inject rule to simulate regional outage in "Second Region"
        is_request_to_second_region: Callable[[HttpRequest], bool] = lambda \
                r: FaultInjectionTransport.predicate_targets_region(r, second_region_uri) and \
                   FaultInjectionTransport.predicate_is_document_operation(r)

        custom_transport.add_fault(
            is_request_to_first_region,
            lambda r: FaultInjectionTransport.error_region_down())
        custom_transport.add_fault(
            is_request_to_second_region,
            lambda r: FaultInjectionTransport.error_region_down())

        id_value: str = str(uuid.uuid4())
        document_definition = {'id': id_value,
                               'pk': id_value,
                               'name': 'sample document',
                               'key': 'value'}

        initialized_objects = self.setup_method_with_custom_transport(
            custom_transport,
            preferred_locations=["First Region", "Second Region"],
            multiple_write_locations=True
        )
        container: ContainerProxy = initialized_objects["col"]
        with pytest.raises(ServiceRequestError):
            container.upsert_item(body=document_definition)


if __name__ == '__main__':
    unittest.main()


# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: sample_delete_tags.py

DESCRIPTION:
    This sample demonstrates deleting all but the most recent three tags for each repository.

USAGE:
    python sample_delete_tags.py

    Set the environment variables with your own values before running the sample:
    1) CONTAINERREGISTRY_ENDPOINT - The URL of your Container Registry account

    This sample assumes your registry has at least one repository with more than three tags,
    run load_registry() if you don't have.
    Set the environment variables with your own values before running load_registry():
    1) CONTAINERREGISTRY_ENDPOINT - The URL of your Container Registry account
    2) CONTAINERREGISTRY_TENANT_ID - The service principal's tenant ID
    3) CONTAINERREGISTRY_CLIENT_ID - The service principal's client ID
    4) CONTAINERREGISTRY_CLIENT_SECRET - The service principal's client secret
"""
import os
from dotenv import find_dotenv, load_dotenv
from azure.containerregistry import ContainerRegistryClient, ArtifactTagOrder
from utilities import load_registry, get_authority, get_credential


class DeleteTags(object):
    def __init__(self):
        load_dotenv(find_dotenv())
        self.endpoint = os.environ["CONTAINERREGISTRY_ENDPOINT"]
        self.authority = get_authority(self.endpoint)
        self.credential = get_credential(self.authority)

    def delete_tags(self):
        load_registry(self.endpoint)
        # [START list_repository_names]
        with ContainerRegistryClient(self.endpoint, self.credential) as client:
            # Iterate through all the repositories
            for repository_name in client.list_repository_names():
                print(repository_name)
                # [END list_repository_names]

                # Keep the three most recent tags, delete everything else
                tag_count = 0
                for tag in client.list_tag_properties(
                    repository_name, order_by=ArtifactTagOrder.LAST_UPDATED_ON_DESCENDING
                ):
                    tag_count += 1
                    if tag_count > 3:
                        print(f"Deleting {repository_name}:{tag.name}")
                        client.delete_tag(repository_name, tag.name)


if __name__ == "__main__":
    sample = DeleteTags()
    sample.delete_tags()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.cosmosdb import CosmosDBManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCosmosDBManagementTableResourcesOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CosmosDBManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_table_resources_list_tables(self, resource_group):
        response = self.client.table_resources.list_tables(
            resource_group_name=resource_group.name,
            account_name="str",
            api_version="2025-04-15",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_table_resources_get_table(self, resource_group):
        response = self.client.table_resources.get_table(
            resource_group_name=resource_group.name,
            account_name="str",
            table_name="str",
            api_version="2025-04-15",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_table_resources_begin_create_update_table(self, resource_group):
        response = self.client.table_resources.begin_create_update_table(
            resource_group_name=resource_group.name,
            account_name="str",
            table_name="str",
            create_update_table_parameters={
                "resource": {
                    "id": "str",
                    "createMode": "Default",
                    "restoreParameters": {
                        "restoreSource": "str",
                        "restoreTimestampInUtc": "2020-02-20 00:00:00",
                        "restoreWithTtlDisabled": bool,
                    },
                },
                "id": "str",
                "location": "str",
                "name": "str",
                "options": {"autoscaleSettings": {"maxThroughput": 0}, "throughput": 0},
                "tags": {"str": "str"},
                "type": "str",
            },
            api_version="2025-04-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_table_resources_begin_delete_table(self, resource_group):
        response = self.client.table_resources.begin_delete_table(
            resource_group_name=resource_group.name,
            account_name="str",
            table_name="str",
            api_version="2025-04-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_table_resources_get_table_throughput(self, resource_group):
        response = self.client.table_resources.get_table_throughput(
            resource_group_name=resource_group.name,
            account_name="str",
            table_name="str",
            api_version="2025-04-15",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_table_resources_begin_update_table_throughput(self, resource_group):
        response = self.client.table_resources.begin_update_table_throughput(
            resource_group_name=resource_group.name,
            account_name="str",
            table_name="str",
            update_throughput_parameters={
                "resource": {
                    "autoscaleSettings": {
                        "maxThroughput": 0,
                        "autoUpgradePolicy": {"throughputPolicy": {"incrementPercent": 0, "isEnabled": bool}},
                        "targetMaxThroughput": 0,
                    },
                    "instantMaximumThroughput": "str",
                    "minimumThroughput": "str",
                    "offerReplacePending": "str",
                    "softAllowedMaximumThroughput": "str",
                    "throughput": 0,
                },
                "id": "str",
                "location": "str",
                "name": "str",
                "tags": {"str": "str"},
                "type": "str",
            },
            api_version="2025-04-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_table_resources_begin_migrate_table_to_autoscale(self, resource_group):
        response = self.client.table_resources.begin_migrate_table_to_autoscale(
            resource_group_name=resource_group.name,
            account_name="str",
            table_name="str",
            api_version="2025-04-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_table_resources_begin_migrate_table_to_manual_throughput(self, resource_group):
        response = self.client.table_resources.begin_migrate_table_to_manual_throughput(
            resource_group_name=resource_group.name,
            account_name="str",
            table_name="str",
            api_version="2025-04-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_table_resources_begin_retrieve_continuous_backup_information(self, resource_group):
        response = self.client.table_resources.begin_retrieve_continuous_backup_information(
            resource_group_name=resource_group.name,
            account_name="str",
            table_name="str",
            location={"location": "str"},
            api_version="2025-04-15",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import Any, AsyncIterable, Callable, Dict, List, Optional, TypeVar

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._digital_twins_operations import build_add_relationship_request, build_add_request, build_delete_relationship_request, build_delete_request, build_get_by_id_request, build_get_component_request, build_get_relationship_by_id_request, build_list_incoming_relationships_request, build_list_relationships_request, build_send_component_telemetry_request, build_send_telemetry_request, build_update_component_request, build_update_relationship_request, build_update_request
T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class DigitalTwinsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.digitaltwins.core.aio.AzureDigitalTwinsAPI`'s
        :attr:`digital_twins` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        args = list(args)
        self._client = args.pop(0) if args else kwargs.pop("client")
        self._config = args.pop(0) if args else kwargs.pop("config")
        self._serialize = args.pop(0) if args else kwargs.pop("serializer")
        self._deserialize = args.pop(0) if args else kwargs.pop("deserializer")


    @distributed_trace_async
    async def get_by_id(
        self,
        id: str,
        digital_twins_get_by_id_options: Optional["_models.DigitalTwinsGetByIdOptions"] = None,
        **kwargs: Any
    ) -> Any:
        """Retrieves a digital twin.
        Status codes:


        * 200 OK
        * 400 Bad Request

          * InvalidArgument - The digital twin id is invalid.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param digital_twins_get_by_id_options: Parameter group. Default value is None.
        :type digital_twins_get_by_id_options:
         ~azure.digitaltwins.core.models.DigitalTwinsGetByIdOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: any, or the result of cls(response)
        :rtype: any
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[Any]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str

        _traceparent = None
        _tracestate = None
        if digital_twins_get_by_id_options is not None:
            _traceparent = digital_twins_get_by_id_options.traceparent
            _tracestate = digital_twins_get_by_id_options.tracestate

        request = build_get_by_id_request(
            id=id,
            api_version=api_version,
            traceparent=_traceparent,
            tracestate=_tracestate,
            template_url=self.get_by_id.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers['ETag']=self._deserialize('str', response.headers.get('ETag'))

        deserialized = self._deserialize('object', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    get_by_id.metadata = {'url': "/digitaltwins/{id}"}  # type: ignore


    @distributed_trace_async
    async def add(
        self,
        id: str,
        twin: Any,
        digital_twins_add_options: Optional["_models.DigitalTwinsAddOptions"] = None,
        **kwargs: Any
    ) -> Optional[Any]:
        """Adds or replaces a digital twin.
        Status codes:


        * 200 OK
        * 400 Bad Request

          * InvalidArgument - The digital twin id or payload is invalid.
          * ModelDecommissioned - The model for the digital twin is decommissioned.
          * TwinLimitReached - The maximum number of digital twins allowed has been reached.
          * ValidationFailed - The digital twin payload is not valid.

        * 412 Precondition Failed

          * PreconditionFailed - The precondition check (If-Match or If-None-Match) failed.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param twin: The digital twin instance being added. If provided, the $dtId property is ignored.
        :type twin: any
        :param digital_twins_add_options: Parameter group. Default value is None.
        :type digital_twins_add_options: ~azure.digitaltwins.core.models.DigitalTwinsAddOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: any, or the result of cls(response)
        :rtype: any or None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[Optional[Any]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _traceparent = None
        _tracestate = None
        _if_none_match = None
        if digital_twins_add_options is not None:
            _traceparent = digital_twins_add_options.traceparent
            _tracestate = digital_twins_add_options.tracestate
            _if_none_match = digital_twins_add_options.if_none_match
        _json = self._serialize.body(twin, 'object')

        request = build_add_request(
            id=id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            traceparent=_traceparent,
            tracestate=_tracestate,
            if_none_match=_if_none_match,
            template_url=self.add.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        deserialized = None
        response_headers = {}
        if response.status_code == 200:
            response_headers['ETag']=self._deserialize('str', response.headers.get('ETag'))
            
            deserialized = self._deserialize('object', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    add.metadata = {'url': "/digitaltwins/{id}"}  # type: ignore


    @distributed_trace_async
    async def delete(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        digital_twins_delete_options: Optional["_models.DigitalTwinsDeleteOptions"] = None,
        **kwargs: Any
    ) -> None:
        """Deletes a digital twin. All relationships referencing the digital twin must already be deleted.
        Status codes:


        * 204 No Content
        * 400 Bad Request

          * InvalidArgument - The digital twin id is invalid.
          * RelationshipsNotDeleted - The digital twin contains relationships.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.

        * 412 Precondition Failed

          * PreconditionFailed - The precondition check (If-Match or If-None-Match) failed.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param digital_twins_delete_options: Parameter group. Default value is None.
        :type digital_twins_delete_options: ~azure.digitaltwins.core.models.DigitalTwinsDeleteOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str

        _traceparent = None
        _tracestate = None
        _if_match = None
        if digital_twins_delete_options is not None:
            _traceparent = digital_twins_delete_options.traceparent
            _tracestate = digital_twins_delete_options.tracestate
            _if_match = digital_twins_delete_options.if_match

        request = build_delete_request(
            id=id,
            api_version=api_version,
            traceparent=_traceparent,
            tracestate=_tracestate,
            if_match=_if_match,
            template_url=self.delete.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        if cls:
            return cls(pipeline_response, None, {})

    delete.metadata = {'url': "/digitaltwins/{id}"}  # type: ignore


    @distributed_trace_async
    async def update(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        patch_document: List[Any],
        digital_twins_update_options: Optional["_models.DigitalTwinsUpdateOptions"] = None,
        **kwargs: Any
    ) -> None:
        """Updates a digital twin.
        Status codes:


        * 204 No Content
        * 400 Bad Request

          * InvalidArgument - The digital twin id or payload is invalid.
          * JsonPatchInvalid - The JSON Patch provided is invalid.
          * ValidationFailed - Applying the patch results in an invalid digital twin.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.

        * 412 Precondition Failed

          * PreconditionFailed - The precondition check (If-Match or If-None-Match) failed.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param patch_document: An update specification described by JSON Patch. Updates to property
         values and $model elements may happen in the same request. Operations are limited to add,
         replace and remove.
        :type patch_document: list[any]
        :param digital_twins_update_options: Parameter group. Default value is None.
        :type digital_twins_update_options: ~azure.digitaltwins.core.models.DigitalTwinsUpdateOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str
        content_type = kwargs.pop('content_type', "application/json-patch+json")  # type: Optional[str]

        _traceparent = None
        _tracestate = None
        _if_match = None
        if digital_twins_update_options is not None:
            _traceparent = digital_twins_update_options.traceparent
            _tracestate = digital_twins_update_options.tracestate
            _if_match = digital_twins_update_options.if_match
        _json = self._serialize.body(patch_document, '[object]')

        request = build_update_request(
            id=id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            traceparent=_traceparent,
            tracestate=_tracestate,
            if_match=_if_match,
            template_url=self.update.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [202, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        if response.status_code == 204:
            response_headers['ETag']=self._deserialize('str', response.headers.get('ETag'))
            

        if cls:
            return cls(pipeline_response, None, response_headers)

    update.metadata = {'url': "/digitaltwins/{id}"}  # type: ignore


    @distributed_trace_async
    async def get_relationship_by_id(
        self,
        id: str,
        relationship_id: str,
        digital_twins_get_relationship_by_id_options: Optional["_models.DigitalTwinsGetRelationshipByIdOptions"] = None,
        **kwargs: Any
    ) -> Any:
        """Retrieves a relationship between two digital twins.
        Status codes:


        * 200 OK
        * 400 Bad Request

          * InvalidArgument - The digital twin id or relationship id is invalid.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.
          * RelationshipNotFound - The relationship was not found.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param relationship_id: The id of the relationship. The id is unique within the digital twin
         and case sensitive.
        :type relationship_id: str
        :param digital_twins_get_relationship_by_id_options: Parameter group. Default value is None.
        :type digital_twins_get_relationship_by_id_options:
         ~azure.digitaltwins.core.models.DigitalTwinsGetRelationshipByIdOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: any, or the result of cls(response)
        :rtype: any
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[Any]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str

        _traceparent = None
        _tracestate = None
        if digital_twins_get_relationship_by_id_options is not None:
            _traceparent = digital_twins_get_relationship_by_id_options.traceparent
            _tracestate = digital_twins_get_relationship_by_id_options.tracestate

        request = build_get_relationship_by_id_request(
            id=id,
            relationship_id=relationship_id,
            api_version=api_version,
            traceparent=_traceparent,
            tracestate=_tracestate,
            template_url=self.get_relationship_by_id.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers['ETag']=self._deserialize('str', response.headers.get('ETag'))

        deserialized = self._deserialize('object', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    get_relationship_by_id.metadata = {'url': "/digitaltwins/{id}/relationships/{relationshipId}"}  # type: ignore


    @distributed_trace_async
    async def add_relationship(
        self,
        id: str,
        relationship_id: str,
        relationship: Any,
        digital_twins_add_relationship_options: Optional["_models.DigitalTwinsAddRelationshipOptions"] = None,
        **kwargs: Any
    ) -> Any:
        """Adds a relationship between two digital twins.
        Status codes:


        * 200 OK
        * 400 Bad Request

          * InvalidArgument - The digital twin id, relationship id, or payload is invalid.
          * InvalidRelationship - The relationship is invalid.
          * OperationNotAllowed - The relationship cannot connect to the same digital twin.
          * ValidationFailed - The relationship content is invalid.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.
          * TargetTwinNotFound - The digital twin target of the relationship was not found.

        * 412 Precondition Failed

          * PreconditionFailed - The precondition check (If-Match or If-None-Match) failed.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param relationship_id: The id of the relationship. The id is unique within the digital twin
         and case sensitive.
        :type relationship_id: str
        :param relationship: The data for the relationship.
        :type relationship: any
        :param digital_twins_add_relationship_options: Parameter group. Default value is None.
        :type digital_twins_add_relationship_options:
         ~azure.digitaltwins.core.models.DigitalTwinsAddRelationshipOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: any, or the result of cls(response)
        :rtype: any
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[Any]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _traceparent = None
        _tracestate = None
        _if_none_match = None
        if digital_twins_add_relationship_options is not None:
            _traceparent = digital_twins_add_relationship_options.traceparent
            _tracestate = digital_twins_add_relationship_options.tracestate
            _if_none_match = digital_twins_add_relationship_options.if_none_match
        _json = self._serialize.body(relationship, 'object')

        request = build_add_relationship_request(
            id=id,
            relationship_id=relationship_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            traceparent=_traceparent,
            tracestate=_tracestate,
            if_none_match=_if_none_match,
            template_url=self.add_relationship.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers['ETag']=self._deserialize('str', response.headers.get('ETag'))

        deserialized = self._deserialize('object', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    add_relationship.metadata = {'url': "/digitaltwins/{id}/relationships/{relationshipId}"}  # type: ignore


    @distributed_trace_async
    async def delete_relationship(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        relationship_id: str,
        digital_twins_delete_relationship_options: Optional["_models.DigitalTwinsDeleteRelationshipOptions"] = None,
        **kwargs: Any
    ) -> None:
        """Deletes a relationship between two digital twins.
        Status codes:


        * 204 No Content
        * 400 Bad Request

          * InvalidArgument - The digital twin id or relationship id is invalid.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.
          * RelationshipNotFound - The relationship was not found.

        * 412 Precondition Failed

          * PreconditionFailed - The precondition check (If-Match or If-None-Match) failed.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param relationship_id: The id of the relationship. The id is unique within the digital twin
         and case sensitive.
        :type relationship_id: str
        :param digital_twins_delete_relationship_options: Parameter group. Default value is None.
        :type digital_twins_delete_relationship_options:
         ~azure.digitaltwins.core.models.DigitalTwinsDeleteRelationshipOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str

        _traceparent = None
        _tracestate = None
        _if_match = None
        if digital_twins_delete_relationship_options is not None:
            _traceparent = digital_twins_delete_relationship_options.traceparent
            _tracestate = digital_twins_delete_relationship_options.tracestate
            _if_match = digital_twins_delete_relationship_options.if_match

        request = build_delete_relationship_request(
            id=id,
            relationship_id=relationship_id,
            api_version=api_version,
            traceparent=_traceparent,
            tracestate=_tracestate,
            if_match=_if_match,
            template_url=self.delete_relationship.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        if cls:
            return cls(pipeline_response, None, {})

    delete_relationship.metadata = {'url': "/digitaltwins/{id}/relationships/{relationshipId}"}  # type: ignore


    @distributed_trace_async
    async def update_relationship(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        relationship_id: str,
        patch_document: List[Any],
        digital_twins_update_relationship_options: Optional["_models.DigitalTwinsUpdateRelationshipOptions"] = None,
        **kwargs: Any
    ) -> None:
        """Updates the properties on a relationship between two digital twins.
        Status codes:


        * 204 No Content
        * 400 Bad Request

          * InvalidArgument - The digital twin id or relationship id is invalid.
          * InvalidRelationship - The relationship is invalid.
          * JsonPatchInvalid - The JSON Patch provided is invalid.
          * ValidationFailed - The relationship content is invalid.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.
          * RelationshipNotFound - The relationship was not found.

        * 409 Conflict

          * RelationshipAlreadyExists - The relationship already exists.

        * 412 Precondition Failed

          * PreconditionFailed - The precondition check (If-Match or If-None-Match) failed.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param relationship_id: The id of the relationship. The id is unique within the digital twin
         and case sensitive.
        :type relationship_id: str
        :param patch_document: JSON Patch description of the update to the relationship properties.
        :type patch_document: list[any]
        :param digital_twins_update_relationship_options: Parameter group. Default value is None.
        :type digital_twins_update_relationship_options:
         ~azure.digitaltwins.core.models.DigitalTwinsUpdateRelationshipOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str
        content_type = kwargs.pop('content_type', "application/json-patch+json")  # type: Optional[str]

        _traceparent = None
        _tracestate = None
        _if_match = None
        if digital_twins_update_relationship_options is not None:
            _traceparent = digital_twins_update_relationship_options.traceparent
            _tracestate = digital_twins_update_relationship_options.tracestate
            _if_match = digital_twins_update_relationship_options.if_match
        _json = self._serialize.body(patch_document, '[object]')

        request = build_update_relationship_request(
            id=id,
            relationship_id=relationship_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            traceparent=_traceparent,
            tracestate=_tracestate,
            if_match=_if_match,
            template_url=self.update_relationship.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers['ETag']=self._deserialize('str', response.headers.get('ETag'))


        if cls:
            return cls(pipeline_response, None, response_headers)

    update_relationship.metadata = {'url': "/digitaltwins/{id}/relationships/{relationshipId}"}  # type: ignore


    @distributed_trace
    def list_relationships(
        self,
        id: str,
        relationship_name: Optional[str] = None,
        digital_twins_list_relationships_options: Optional["_models.DigitalTwinsListRelationshipsOptions"] = None,
        **kwargs: Any
    ) -> AsyncIterable["_models.RelationshipCollection"]:
        """Retrieves the relationships from a digital twin.
        Status codes:


        * 200 OK
        * 400 Bad Request

          * InvalidArgument - The digital twin id is invalid.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param relationship_name: The name of the relationship. Default value is None.
        :type relationship_name: str
        :param digital_twins_list_relationships_options: Parameter group. Default value is None.
        :type digital_twins_list_relationships_options:
         ~azure.digitaltwins.core.models.DigitalTwinsListRelationshipsOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either RelationshipCollection or the result of
         cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.digitaltwins.core.models.RelationshipCollection]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str

        cls = kwargs.pop('cls', None)  # type: ClsType["_models.RelationshipCollection"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                _traceparent = None
                _tracestate = None
                if digital_twins_list_relationships_options is not None:
                    _traceparent = digital_twins_list_relationships_options.traceparent
                    _tracestate = digital_twins_list_relationships_options.tracestate
                
                request = build_list_relationships_request(
                    id=id,
                    api_version=api_version,
                    traceparent=_traceparent,
                    tracestate=_tracestate,
                    relationship_name=relationship_name,
                    template_url=self.list_relationships.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                _traceparent = None
                _tracestate = None
                if digital_twins_list_relationships_options is not None:
                    _traceparent = digital_twins_list_relationships_options.traceparent
                    _tracestate = digital_twins_list_relationships_options.tracestate
                
                request = build_list_relationships_request(
                    id=id,
                    api_version=api_version,
                    traceparent=_traceparent,
                    tracestate=_tracestate,
                    relationship_name=relationship_name,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("RelationshipCollection", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
                request,
                stream=False,
                **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error)

            return pipeline_response


        return AsyncItemPaged(
            get_next, extract_data
        )
    list_relationships.metadata = {'url': "/digitaltwins/{id}/relationships"}  # type: ignore

    @distributed_trace
    def list_incoming_relationships(
        self,
        id: str,
        digital_twins_list_incoming_relationships_options: Optional["_models.DigitalTwinsListIncomingRelationshipsOptions"] = None,
        **kwargs: Any
    ) -> AsyncIterable["_models.IncomingRelationshipCollection"]:
        """Retrieves all incoming relationship for a digital twin.
        Status codes:


        * 200 OK
        * 400 Bad Request

          * InvalidArgument - The digital twin id is invalid.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param digital_twins_list_incoming_relationships_options: Parameter group. Default value is
         None.
        :type digital_twins_list_incoming_relationships_options:
         ~azure.digitaltwins.core.models.DigitalTwinsListIncomingRelationshipsOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either IncomingRelationshipCollection or the result of
         cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.digitaltwins.core.models.IncomingRelationshipCollection]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str

        cls = kwargs.pop('cls', None)  # type: ClsType["_models.IncomingRelationshipCollection"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                _traceparent = None
                _tracestate = None
                if digital_twins_list_incoming_relationships_options is not None:
                    _traceparent = digital_twins_list_incoming_relationships_options.traceparent
                    _tracestate = digital_twins_list_incoming_relationships_options.tracestate
                
                request = build_list_incoming_relationships_request(
                    id=id,
                    api_version=api_version,
                    traceparent=_traceparent,
                    tracestate=_tracestate,
                    template_url=self.list_incoming_relationships.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                _traceparent = None
                _tracestate = None
                if digital_twins_list_incoming_relationships_options is not None:
                    _traceparent = digital_twins_list_incoming_relationships_options.traceparent
                    _tracestate = digital_twins_list_incoming_relationships_options.tracestate
                
                request = build_list_incoming_relationships_request(
                    id=id,
                    api_version=api_version,
                    traceparent=_traceparent,
                    tracestate=_tracestate,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("IncomingRelationshipCollection", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
                request,
                stream=False,
                **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error)

            return pipeline_response


        return AsyncItemPaged(
            get_next, extract_data
        )
    list_incoming_relationships.metadata = {'url': "/digitaltwins/{id}/incomingrelationships"}  # type: ignore

    @distributed_trace_async
    async def send_telemetry(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        message_id: str,
        telemetry: Any,
        telemetry_source_time: Optional[str] = None,
        digital_twins_send_telemetry_options: Optional["_models.DigitalTwinsSendTelemetryOptions"] = None,
        **kwargs: Any
    ) -> None:
        """Sends telemetry on behalf of a digital twin.
        Status codes:


        * 204 No Content
        * 400 Bad Request

          * InvalidArgument - The digital twin id or message id is invalid.
          * ValidationFailed - The telemetry content is invalid.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param message_id: A unique message identifier (in the scope of the digital twin id) that is
         commonly used for de-duplicating messages.
        :type message_id: str
        :param telemetry: The telemetry measurements to send from the digital twin.
        :type telemetry: any
        :param telemetry_source_time: An RFC 3339 timestamp that identifies the time the telemetry was
         measured. Default value is None.
        :type telemetry_source_time: str
        :param digital_twins_send_telemetry_options: Parameter group. Default value is None.
        :type digital_twins_send_telemetry_options:
         ~azure.digitaltwins.core.models.DigitalTwinsSendTelemetryOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _traceparent = None
        _tracestate = None
        if digital_twins_send_telemetry_options is not None:
            _traceparent = digital_twins_send_telemetry_options.traceparent
            _tracestate = digital_twins_send_telemetry_options.tracestate
        _json = self._serialize.body(telemetry, 'object')

        request = build_send_telemetry_request(
            id=id,
            api_version=api_version,
            content_type=content_type,
            message_id=message_id,
            json=_json,
            traceparent=_traceparent,
            tracestate=_tracestate,
            telemetry_source_time=telemetry_source_time,
            template_url=self.send_telemetry.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        if cls:
            return cls(pipeline_response, None, {})

    send_telemetry.metadata = {'url': "/digitaltwins/{id}/telemetry"}  # type: ignore


    @distributed_trace_async
    async def send_component_telemetry(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        component_path: str,
        message_id: str,
        telemetry: Any,
        telemetry_source_time: Optional[str] = None,
        digital_twins_send_component_telemetry_options: Optional["_models.DigitalTwinsSendComponentTelemetryOptions"] = None,
        **kwargs: Any
    ) -> None:
        """Sends telemetry on behalf of a component in a digital twin.
        Status codes:


        * 204 No Content
        * 400 Bad Request

          * InvalidArgument - The digital twin id, message id, or component path is invalid.
          * ValidationFailed - The telemetry content is invalid.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.
          * ComponentNotFound - The component path was not found.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param component_path: The name of the DTDL component.
        :type component_path: str
        :param message_id: A unique message identifier (in the scope of the digital twin id) that is
         commonly used for de-duplicating messages.
        :type message_id: str
        :param telemetry: The telemetry measurements to send from the digital twin's component.
        :type telemetry: any
        :param telemetry_source_time: An RFC 3339 timestamp that identifies the time the telemetry was
         measured. Default value is None.
        :type telemetry_source_time: str
        :param digital_twins_send_component_telemetry_options: Parameter group. Default value is None.
        :type digital_twins_send_component_telemetry_options:
         ~azure.digitaltwins.core.models.DigitalTwinsSendComponentTelemetryOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _traceparent = None
        _tracestate = None
        if digital_twins_send_component_telemetry_options is not None:
            _traceparent = digital_twins_send_component_telemetry_options.traceparent
            _tracestate = digital_twins_send_component_telemetry_options.tracestate
        _json = self._serialize.body(telemetry, 'object')

        request = build_send_component_telemetry_request(
            id=id,
            component_path=component_path,
            api_version=api_version,
            content_type=content_type,
            message_id=message_id,
            json=_json,
            traceparent=_traceparent,
            tracestate=_tracestate,
            telemetry_source_time=telemetry_source_time,
            template_url=self.send_component_telemetry.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        if cls:
            return cls(pipeline_response, None, {})

    send_component_telemetry.metadata = {'url': "/digitaltwins/{id}/components/{componentPath}/telemetry"}  # type: ignore


    @distributed_trace_async
    async def get_component(
        self,
        id: str,
        component_path: str,
        digital_twins_get_component_options: Optional["_models.DigitalTwinsGetComponentOptions"] = None,
        **kwargs: Any
    ) -> Any:
        """Retrieves a component from a digital twin.
        Status codes:


        * 200 OK
        * 400 Bad Request

          * InvalidArgument - The digital twin id or component path is invalid.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.
          * ComponentNotFound - The component path was not found.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param component_path: The name of the DTDL component.
        :type component_path: str
        :param digital_twins_get_component_options: Parameter group. Default value is None.
        :type digital_twins_get_component_options:
         ~azure.digitaltwins.core.models.DigitalTwinsGetComponentOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: any, or the result of cls(response)
        :rtype: any
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[Any]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str

        _traceparent = None
        _tracestate = None
        if digital_twins_get_component_options is not None:
            _traceparent = digital_twins_get_component_options.traceparent
            _tracestate = digital_twins_get_component_options.tracestate

        request = build_get_component_request(
            id=id,
            component_path=component_path,
            api_version=api_version,
            traceparent=_traceparent,
            tracestate=_tracestate,
            template_url=self.get_component.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers['ETag']=self._deserialize('str', response.headers.get('ETag'))

        deserialized = self._deserialize('object', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    get_component.metadata = {'url': "/digitaltwins/{id}/components/{componentPath}"}  # type: ignore


    @distributed_trace_async
    async def update_component(  # pylint: disable=inconsistent-return-statements
        self,
        id: str,
        component_path: str,
        patch_document: List[Any],
        digital_twins_update_component_options: Optional["_models.DigitalTwinsUpdateComponentOptions"] = None,
        **kwargs: Any
    ) -> None:
        """Updates a component on a digital twin.
        Status codes:


        * 204 No Content
        * 400 Bad Request

          * InvalidArgument - The digital twin id, component path, or payload is invalid.
          * JsonPatchInvalid - The JSON Patch provided is invalid.
          * ValidationFailed - Applying the patch results in an invalid digital twin.

        * 404 Not Found

          * DigitalTwinNotFound - The digital twin was not found.

        * 412 Precondition Failed

          * PreconditionFailed - The precondition check (If-Match or If-None-Match) failed.

        :param id: The id of the digital twin. The id is unique within the service and case sensitive.
        :type id: str
        :param component_path: The name of the DTDL component.
        :type component_path: str
        :param patch_document: An update specification described by JSON Patch. Updates to property
         values and $model elements may happen in the same request. Operations are limited to add,
         replace and remove.
        :type patch_document: list[any]
        :param digital_twins_update_component_options: Parameter group. Default value is None.
        :type digital_twins_update_component_options:
         ~azure.digitaltwins.core.models.DigitalTwinsUpdateComponentOptions
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2022-05-31")  # type: str
        content_type = kwargs.pop('content_type', "application/json-patch+json")  # type: Optional[str]

        _traceparent = None
        _tracestate = None
        _if_match = None
        if digital_twins_update_component_options is not None:
            _traceparent = digital_twins_update_component_options.traceparent
            _tracestate = digital_twins_update_component_options.tracestate
            _if_match = digital_twins_update_component_options.if_match
        _json = self._serialize.body(patch_document, '[object]')

        request = build_update_component_request(
            id=id,
            component_path=component_path,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            traceparent=_traceparent,
            tracestate=_tracestate,
            if_match=_if_match,
            template_url=self.update_component.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [202, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        if response.status_code == 204:
            response_headers['ETag']=self._deserialize('str', response.headers.get('ETag'))
            

        if cls:
            return cls(pipeline_response, None, response_headers)

    update_component.metadata = {'url': "/digitaltwins/{id}/components/{componentPath}"}  # type: ignore




# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import sys
from typing import Any, Union, IO
from azure.core.credentials import AzureKeyCredential
from azure.core.credentials_async import AsyncTokenCredential
from azure.core.polling import AsyncLROPoller
from azure.core.tracing.decorator_async import distributed_trace_async
from .._shared.auth_policy_utils import get_authentication_policy
from .._shared.utils import parse_connection_str
from .._generated.aio._client import AzureCommunicationEmailService
from .._version import SDK_MONIKER
from .._api_versions import DEFAULT_VERSION

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object


class EmailClient(object):  # pylint: disable=client-accepts-api-version-keyword
    """A client to interact with the AzureCommunicationService Email gateway asynchronously.

    This client provides operations to send an email and monitor its status.

    :param str endpoint:
        The endpoint url for Azure Communication Service resource.
    :param Union[AsyncTokenCredential, AzureKeyCredential] credential:
        The credential we use to authenticate against the service.
    :keyword api_version: Azure Communication Email API version.
        Default value is "2024-07-01-preview".
        Note that overriding this default value may result in unsupported behavior.
    :paramtype api_version: str
    """

    def __init__(self, endpoint: str, credential: Union[AsyncTokenCredential, AzureKeyCredential], **kwargs) -> None:
        try:
            if not endpoint.lower().startswith("http"):
                endpoint = "https://" + endpoint
        except AttributeError:
            raise ValueError("Account URL must be a string.")  # pylint: disable=raise-missing-from

        if endpoint.endswith("/"):
            endpoint = endpoint[:-1]

        self._api_version = kwargs.pop("api_version", DEFAULT_VERSION)

        authentication_policy = get_authentication_policy(endpoint, credential, decode_url=True, is_async=True)

        self._generated_client = AzureCommunicationEmailService(
            endpoint, authentication_policy=authentication_policy, sdk_moniker=SDK_MONIKER, **kwargs
        )

    @classmethod
    def from_connection_string(cls, conn_str: str, **kwargs) -> "EmailClient":
        """Create EmailClient from a Connection String.

        :param str conn_str:
            A connection string to an Azure Communication Service resource.
        :returns: Instance of EmailClient.
        :rtype: ~azure.communication.EmailClient
        """
        endpoint, access_key = parse_connection_str(conn_str)

        return cls(endpoint, AzureKeyCredential(access_key), **kwargs)

    @distributed_trace_async
    async def begin_send(self, message: Union[JSON, IO], **kwargs: Any) -> AsyncLROPoller[JSON]:
        # cSpell:disable
        """Queues an email message to be sent to one or more recipients.

        Queues an email message to be sent to one or more recipients.

        :param message: Message payload for sending an email. Required.
        :type message: JSON
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :return: An instance of LROPoller that returns JSON object
        :rtype: ~azure.core.polling.LROPoller[JSON]
        :raises ~azure.core.exceptions.HttpResponseError:

         Example:
            .. code-block:: python

                # JSON input template you can fill out and use as your body input.
                message = {
                    "content": {
                        "subject": "str",  # Subject of the email message. Required.
                        "html": "str",  # Optional. Html version of the email message.
                        "plainText": "str"  # Optional. Plain text version of the email
                          message.
                    },
                    "recipients": {
                        "to": [
                            {
                                "address": "str",  # Email address. Required.
                                "displayName": "str"  # Optional. Email display name.
                            }
                        ],
                        "bcc": [
                            {
                                "address": "str",  # Email address. Required.
                                "displayName": "str"  # Optional. Email display name.
                            }
                        ],
                        "cc": [
                            {
                                "address": "str",  # Email address. Required.
                                "displayName": "str"  # Optional. Email display name.
                            }
                        ]
                    },
                    "senderAddress": "str",  # Sender email address from a verified domain.
                      Required.
                    "attachments": [
                        {
                            "contentInBase64": "str",  # Base64 encoded contents of the
                              attachment. Required.
                            "contentType": "str",  # MIME type of the content being
                              attached. Required.
                            "name": "str",  # Name of the attachment. Required.
                            "contentId": "str"  # Optional. Unique identifier (CID) to
                              reference an inline attachment.
                        }
                    ],
                    "userEngagementTrackingDisabled": bool,  # Optional. Indicates whether user
                      engagement tracking should be disabled for this request if the resource-level
                      user engagement tracking setting was already enabled in the control plane.
                    "headers": {
                        "str": "str"  # Optional. Custom email headers to be passed.
                    },
                    "replyTo": [
                        {
                            "address": "str",  # Email address. Required.
                            "displayName": "str"  # Optional. Email display name.
                        }
                    ]
                }

                # response body for status code(s): 202
                response == {
                    "id": "str",  # The unique id of the operation. Use a UUID. Required.
                    "status": "str",  # Status of operation. Required. Known values are:
                      "NotStarted", "Running", "Succeeded", "Failed", and "Canceled".
                    "error": {
                        "additionalInfo": [
                            {
                                "info": {},  # Optional. The additional info.
                                "type": "str"  # Optional. The additional info type.
                            }
                        ],
                        "code": "str",  # Optional. The error code.
                        "details": [
                            ...
                        ],
                        "message": "str",  # Optional. The error message.
                        "target": "str"  # Optional. The error target.
                    }
                }
        """
        # cSpell:enable

        return await self._generated_client.email.begin_send(message=message, **kwargs)

    async def __aenter__(self) -> "EmailClient":
        await self._generated_client.__aenter__()
        return self

    async def __aexit__(self, *args) -> None:
        await self._generated_client.__aexit__(*args)

    async def close(self) -> None:
        await self._generated_client.close()



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import unittest
import pytest
import azure.mgmt.devtestlabs
from devtools_testutils import AzureMgmtRecordedTestCase, ResourceGroupPreparer, recorded_by_proxy


@pytest.mark.live_test_only
class TestMgmtDevTestLabs(AzureMgmtRecordedTestCase):

    def setup_method(self, method):
        self.client = self.create_mgmt_client(azure.mgmt.devtestlabs.DevTestLabsClient)

    @ResourceGroupPreparer()
    @recorded_by_proxy
    def test_devtestlabs(self, resource_group, location):
        lab_name = self.get_resource_name("pylab")

        async_lab = self.client.labs.begin_create_or_update(resource_group.name, lab_name, {"location": location})
        lab = async_lab.result()
        assert lab.name == lab_name

        # get
        self.client.labs.get(resource_group.name, lab_name)

        # update
        BODY = {"properties": {"labStorageType": "Premium"}}
        self.client.labs.update(resource_group.name, lab_name, BODY)

        # delete
        result = self.client.labs.begin_delete(resource_group.name, lab_name)
        result.result()


# ------------------------------------------------------------------------------
if __name__ == "__main__":
    unittest.main()



# --------------------------------------------------------------------------
#
# Copyright (c) Microsoft Corporation. All rights reserved.
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the ""Software""), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
# --------------------------------------------------------------------------
from typing import TypeVar, AsyncContextManager, Any
from collections.abc import MutableSequence
from azure.core import AsyncPipelineClient
from .policies import (
    AsyncARMAutoResourceProviderRegistrationPolicy,
    ARMHttpLoggingPolicy,
)

HTTPRequestType = TypeVar("HTTPRequestType")
AsyncHTTPResponseType = TypeVar("AsyncHTTPResponseType", bound=AsyncContextManager)


class AsyncARMPipelineClient(AsyncPipelineClient[HTTPRequestType, AsyncHTTPResponseType]):
    """A pipeline client designed for ARM explicitly.

    :param str base_url: URL for the request.
    :keyword AsyncPipeline pipeline: If omitted, a Pipeline object is created and returned.
    :keyword list[AsyncHTTPPolicy] policies: If omitted, the standard policies of the configuration object is used.
    :keyword per_call_policies: If specified, the policies will be added into the policy list before RetryPolicy
    :paramtype per_call_policies: Union[AsyncHTTPPolicy, SansIOHTTPPolicy,
        list[AsyncHTTPPolicy], list[SansIOHTTPPolicy]]
    :keyword per_retry_policies: If specified, the policies will be added into the policy list after RetryPolicy
    :paramtype per_retry_policies: Union[AsyncHTTPPolicy, SansIOHTTPPolicy,
        list[AsyncHTTPPolicy], list[SansIOHTTPPolicy]]
    :keyword AsyncHttpTransport transport: If omitted, AioHttpTransport is used for asynchronous transport.
    """

    def __init__(self, base_url: str, **kwargs: Any):
        if "policies" not in kwargs:
            config = kwargs.get("config")
            if not config:
                raise ValueError("Current implementation requires to pass 'config' if you don't pass 'policies'")
            per_call_policies = kwargs.get("per_call_policies", [])
            if isinstance(per_call_policies, MutableSequence):
                per_call_policies.append(AsyncARMAutoResourceProviderRegistrationPolicy())
            else:
                per_call_policies = [
                    per_call_policies,
                    AsyncARMAutoResourceProviderRegistrationPolicy(),
                ]
            kwargs["per_call_policies"] = per_call_policies
            if not config.http_logging_policy:
                config.http_logging_policy = kwargs.get("http_logging_policy", ARMHttpLoggingPolicy(**kwargs))
            kwargs["config"] = config
        super(AsyncARMPipelineClient, self).__init__(base_url, **kwargs)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import functools
from devtools_testutils import PowerShellPreparer

CommunicationPreparer = functools.partial(
    PowerShellPreparer,
    "communication",
    communication_livetest_dynamic_connection_string="endpoint=https://sanitized.communication.azure.com/;accesskey=fake===",
)
